@@ -33,7 +33,6 @@
 #include "ev-job-queue.h"
 #include "ev-document-links.h"
 #include "ev-window.h"
-#include "ev-gui.h"
 
 struct _EvSidebarLinksPrivate {
 	GtkWidget *tree_view;
@@ -55,6 +54,10 @@ enum {
 	PROP_WIDGET,
 };
 
+enum {
+	LINK_ACTIVATED,
+	N_SIGNALS
+};
 
 static void links_page_num_func				(GtkTreeViewColumn *tree_column,
 							 GtkCellRenderer   *cell,
@@ -77,6 +80,7 @@ static gboolean ev_sidebar_links_support_document	(EvSidebarPage  *sidebar_page,
 						         EvDocument     *document);
 static const gchar* ev_sidebar_links_get_label 		(EvSidebarPage *sidebar_page);
 
+static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarLinks, 
                         ev_sidebar_links, 
@@ -177,6 +181,14 @@ ev_sidebar_links_class_init (EvSidebarLinksClass *ev_sidebar_links_class)
 	g_object_class->get_property = ev_sidebar_links_get_property;
 	g_object_class->dispose = ev_sidebar_links_dispose;
 
+	signals[LINK_ACTIVATED] = g_signal_new ("link-activated",
+			 G_TYPE_FROM_CLASS (g_object_class),
+		         G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		         G_STRUCT_OFFSET (EvSidebarLinksClass, link_activated),
+		         NULL, NULL,
+		         g_cclosure_marshal_VOID__OBJECT,
+		         G_TYPE_NONE, 1, G_TYPE_OBJECT);
+
 	g_object_class_install_property (g_object_class,
 					 PROP_MODEL,
 					 g_param_spec_object ("model",
@@ -216,8 +228,7 @@ selection_changed_callback (GtkTreeSelection   *selection,
 
 		g_signal_handler_block (ev_sidebar_links->priv->page_cache,
 					ev_sidebar_links->priv->page_changed_id);
-		/* FIXME: we should handle this better.  This breaks w/ URLs */
-		ev_page_cache_set_link (ev_sidebar_links->priv->page_cache, link);
+		g_signal_emit (ev_sidebar_links, signals[LINK_ACTIVATED], 0, link);
 		g_signal_handler_unblock (ev_sidebar_links->priv->page_cache,
 					  ev_sidebar_links->priv->page_changed_id);
 
@@ -250,6 +261,27 @@ create_loading_model (void)
 	return retval;
 }
 
+static gint
+get_page_from_link (EvLink *link)
+{
+	EvLinkAction *action;
+	EvLinkDest *dest;
+
+	action = ev_link_get_action (link);
+	if (!action)
+		return -1;
+
+	if (ev_link_action_get_action_type (action) !=
+	    EV_LINK_ACTION_TYPE_GOTO_DEST)
+		return -1;
+
+	dest = ev_link_action_get_dest (action);
+	if (dest)
+		return ev_link_dest_get_page (dest);
+		
+	return -1;
+}
+
 static void
 print_section_cb (GtkWidget *menuitem, EvSidebarLinks *sidebar)
 {
@@ -263,32 +295,46 @@ print_section_cb (GtkWidget *menuitem, EvSidebarLinks *sidebar)
 
 	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		EvLink *link;
-		int first_page, last_page;
+		int first_page, last_page = -1;
 
 		gtk_tree_model_get (model, &iter,
 				    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 				    -1);
-		first_page = ev_link_get_page (link) + 1;
 
-		if (link)
+		if (!link)
+			return;
+
+		first_page = get_page_from_link (link);
+		if (first_page == -1) {
 			g_object_unref (link);
+			return;
+		}
+		
+		first_page++;
+		g_object_unref (link);
 
 		if (gtk_tree_model_iter_next (model, &iter)) {
 			gtk_tree_model_get (model, &iter,
 					    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 					    -1);
-			last_page = ev_link_get_page (link);
 
-			if (link)
-		    		g_object_unref (link);
+			if (link) {
+				last_page = get_page_from_link (link);
+				g_object_unref (link);
+			}
 		} else {
-			last_page = -1;
+			last_page = ev_page_cache_get_n_pages (sidebar->priv->page_cache);
 		}
+
+		if (last_page == -1)
+			last_page = ev_page_cache_get_n_pages (sidebar->priv->page_cache);
 	
 		window = gtk_widget_get_toplevel (GTK_WIDGET (sidebar));
 		if (EV_IS_WINDOW (window)) {
+#ifdef WITH_PRINT
 			ev_window_print_range (EV_WINDOW (window),
 					       first_page, last_page);
+#endif
 		}
 	}
 }
@@ -428,17 +474,27 @@ links_page_num_func (GtkTreeViewColumn *tree_column,
 		     EvSidebarLinks    *sidebar_links)
 {
 	EvLink *link;
+	gint page;
 
 	gtk_tree_model_get (tree_model, iter,
 			    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 			    -1);
-	
-	if (link != NULL &&
-	    ev_link_get_link_type (link) == EV_LINK_TYPE_PAGE) {
+
+	if (!link) {
+		g_object_set (cell,
+			      "visible", FALSE,
+			      NULL);
+		return;
+	}
+
+	page = get_page_from_link (link);
+
+	if (page >= 0) {
 		gchar *page_label;
 		gchar *page_string;
 
-		page_label = ev_page_cache_get_page_label (sidebar_links->priv->page_cache, ev_link_get_page (link));
+		page_label = ev_page_cache_get_page_label (sidebar_links->priv->page_cache,
+							   page);
 		page_string = g_markup_printf_escaped ("<i>%s</i>", page_label);
 
 		g_object_set (cell,
@@ -454,8 +510,7 @@ links_page_num_func (GtkTreeViewColumn *tree_column,
 			      NULL);
 	}
 
-	if (link)
-		g_object_unref (link);
+	g_object_unref (link);
 }
 
 /* Public Functions */
@@ -483,25 +538,27 @@ update_page_callback_foreach (GtkTreeModel *model,
 			    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 			    -1);
 
-	if (link && ev_link_get_link_type (link) == EV_LINK_TYPE_PAGE) {
+	if (link) {
 		int current_page;
+		int dest_page;
 
+		dest_page = get_page_from_link (link);
+		g_object_unref (link);
+		
 		current_page = ev_page_cache_get_current_page (sidebar_links->priv->page_cache);
-		if (ev_link_get_page (link) == current_page) {
+			 
+		if (dest_page == current_page) {
 			GtkTreeSelection *selection;
 
 			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
-
+			gtk_tree_view_expand_to_path (GTK_TREE_VIEW (sidebar_links->priv->tree_view),
+						      path);
 			gtk_tree_selection_select_path (selection, path);
-	
-			g_object_unref (link);
+			
 			return TRUE;
 		}
 	}
 
-	if (link)
-		g_object_unref (link);
-	
 	return FALSE;
 }
 
@@ -511,21 +568,42 @@ update_page_callback (EvPageCache    *page_cache,
 		      EvSidebarLinks *sidebar_links)
 {
 	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		EvLink *link;
+
+		gtk_tree_model_get (model, &iter,
+				    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
+				    -1);
+		if (link) {
+			gint current_page;
+			gint dest_page;
+
+			dest_page = get_page_from_link (link);
+			g_object_unref (link);
+			
+			current_page = ev_page_cache_get_current_page (sidebar_links->priv->page_cache);
+			if (dest_page == current_page)
+				return;
+		}
+	}		
+
 	/* We go through the tree linearly looking for the first page that
 	 * matches.  This is pretty inefficient.  We can do something neat with
 	 * a GtkTreeModelSort here to make it faster, if it turns out to be
 	 * slow.
 	 */
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
-
 	g_signal_handler_block (selection, sidebar_links->priv->selection_id);
 	g_signal_handler_block (sidebar_links->priv->tree_view, sidebar_links->priv->row_activated_id);
 
-	gtk_tree_selection_unselect_all (selection);
-	gtk_tree_model_foreach (sidebar_links->priv->model,
+	gtk_tree_model_foreach (model,
 				update_page_callback_foreach,
 				sidebar_links);
+	
 
 	g_signal_handler_unblock (selection, sidebar_links->priv->selection_id);
 	g_signal_handler_unblock (sidebar_links->priv->tree_view, sidebar_links->priv->row_activated_id);
